<?php

/**
 * @file
 * The forms used by Plugin Manager.
 */

/**
 * FAPI for overview/discovery page.
 */
function plugin_manager_discover_form($form_state) {

  // If the repository is over 24 hours old then update it.
  if (variable_get('plugin_manager_last_reload', 0) + 604800 < date('U')) {
    drupal_set_message(l(t('The repository is out of date.  Update it by clicking here.'), 'admin/config/system/plugin_manager/reload'));
  }

  $form['discover_box'] = array(
    '#title' => t('Discover'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // Search by name and/or filter by type.
  $form['discover_box']['search_box'] = array(
    '#title' => t('Search'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['discover_box']['search_box']['search'] = array(
    '#title' => t('Search by name'),
    '#type' => 'textfield',
  );
  $form['discover_box']['search_box']['filter'] = array(
    '#title' => t('Filter by type'),
    '#type' => 'select',
    '#options' => array(
      'None' => t('Type:'),
      'Modules' => t('Modules'),
      'Themes' => t('Themes'),
    ),
  );

  // Browse by category.
  $form['discover_box']['browse_box'] = array(
    '#title' => t('Browse'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $invalid_tags = array(
    'Actively maintained',
    'Distributions',
    'Minimally maintained',
    'Modules',
    'Themes',
    'Translations',
  );
  $options = array('None' => 'Category:');
  $query = "SELECT tag FROM {plugin_manager_taxonomy} WHERE tag NOT IN('" . implode("','", $invalid_tags) . "') GROUP BY tag ORDER BY tag";
  $categories = db_query($query);
  foreach ($categories as $category) {
    $options[$category->tag] = t($category->tag);
  }
  $form['discover_box']['browse_box']['category'] = array(
    '#title' => t('Browse by category'),
    '#type' => 'select',
    '#options' => $options,
  );

  $form['discover_box']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Search'),
  );

  // Get a list of queued changes.
  $queue = variable_get('plugin_manager_queue', array());

  // Make a list that shows the queue.
  if (!empty($queue)) {

    // Get a list of all queued items.
    $form['queue_box'] = array(
      '#type' => 'fieldset',
      '#title' => t('Queue'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    foreach ($queue as $item) {
      $title = db_query('SELECT title FROM {plugin_manager_repository} WHERE short_name = :short_name', array(':short_name' => $item))->fetchField();

      $form['queue_box']['plugin_' . $item] = array(
        '#type' => 'checkbox',
        '#name' => 'plugin_' . $item,
        '#title' => check_plain($title),
        '#default_value' => TRUE,
      );
    }
    $form['queue_box']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Install'),
    );
    $form['queue_box']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Clear'),
    );
  }

  // Get a list of search results.
  $results = variable_get('plugin_manager_results', array());

  // If we are searching for something.
  if (!empty($results)) {
    $form['discover_box']['#collapsed'] = TRUE;

    // Get a list of all of the tags.
    $form['result_box'] = array(
      '#type' => 'fieldset',
      '#title' => t('Results'),
      '#collapsible' => TRUE,
      '#collapsed' => !empty($queue),
    );

    $form['result_box']['message'] = array(
      '#title' => '',
      '#type' => 'item',
      '#markup' => t('For more information, click the project title to open in a new tab.'),
    );

    $result_count = 0;
    foreach ($results as $short_name => $title) {
      $form['result_box']['plugin_' . $short_name] = array(
        '#type' => 'checkbox',
        '#name' => 'plugin_' . $short_name,
        '#title' => $title,
        '#default_value' => isset($queue[$short_name]),
      );
      $result_count++;
    }

    $form['result_box']['count'] = array(
      '#title' => '',
      '#type' => 'item',
      '#markup' => ($result_count == 1) ? t('1 project found') : $result_count . ' ' . t('projects found'),
    );

    $form['result_box']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Queue for Installation'),
    );
  }

  return $form;
}

/**
 * Plugin Manager discover form validation handler.
 */
function plugin_manager_discover_form_validate($form, &$form_state) {
  $clicked = $form_state['clicked_button']['#value'];

  if ($clicked == 'Search') {

    // Make sure search criteria exists.  We don't want to return all possible results to keep resources manageable.
    if (empty($form_state['values']['search']) && $form_state['values']['filter'] == 'None' && $form_state['values']['category'] == 'None') {
      form_set_error('search', t('No search criteria.'));
    }

    // Make sure the search criteria is not too broad.  We still don't want to return too many results to keep resources manageable.
    if (empty($form_state['values']['search']) && $form_state['values']['filter'] == 'Modules' && $form_state['values']['category'] == 'None') {
      form_set_error('filter', t('Search criteria too broad.  Please try a more granular search.'));
    }
  }

}

/**
 * Plugin Manager discover form submit handler.
 */
function plugin_manager_discover_form_submit($form, &$form_state) {
  // Clear out any previously selected versions.
  variable_set('plugin_manager_versions', array());

  // Determine which submit button was clicked.
  $clicked = $form_state['clicked_button']['#value'];

  if ($clicked == 'Clear') {
    variable_set('plugin_manager_queue', array());
    drupal_goto('admin/config/system/plugin_manager');
  }

  if ($clicked == 'Search') {

    $results = array();

    // Build the where clause.
    $where = array('1');
    $args = array();

    // If the search string exists, then add it to our where clause.
    if (!empty($form_state['values']['search'])) {
      $where[] = 'title LIKE :title';
      $args[':title'] = '%' . $form_state['values']['search'] . '%';
    }

    // See if a filter has been selected.
    if ($form_state['values']['filter'] != 'None') {
      $where[] = 'tag = :tag';
      $args[':tag'] = $form_state['values']['filter'];
    }
    elseif ($form_state['values']['category'] != 'None') {
      $where[] = 'tag = :tag';
      $args[':tag'] = $form_state['values']['category'];
    }

    $where = implode(' AND ', $where);

    // Get a list of the non-installed plugins from the category.
    // @todo DBTNG this
    $plugins = db_query(
       "SELECT title, pmr.short_name AS short_name, context, url
        FROM {plugin_manager_repository} pmr
        LEFT JOIN {plugin_manager_taxonomy} pmt
        ON (pmt.short_name = pmr.short_name)
        WHERE $where
        ORDER BY title", $args
    );

    foreach ($plugins as $plugin) {
      $plugin_dir = drupal_get_path($plugin->context, $plugin->short_name);
      if (!is_dir($plugin_dir)) {
        $results[$plugin->short_name] = l(check_plain($plugin->title), $plugin->url, array(
          'html' => TRUE,
          'attributes' => array('target' => '_blank'),
        ));
      }
      else {
        unset($results[$plugin->short_name]);
      }
    }

    variable_set('plugin_manager_results', $results);
  }

  // See what is already queued.
  $queue = variable_get('plugin_manager_queue', array());

  // Go through each value in the list.
  foreach ($form_state['values'] as $plugin => $value) {
    if (substr($plugin, 0, 7) == 'plugin_') {
      $plugin = substr($plugin, 7);
      $plugin_dir = drupal_get_path('module', $plugin);
      if (!is_dir($plugin_dir) && $value) {
        // Add it to the queue if needed.
        $queue[$plugin] = $plugin;
      }
      else {
        // Remove it from the queue otherwise.
        unset($queue[$plugin]);
      }
    }
  }

  // Set the new queue equal to our changed queue.
  variable_set('plugin_manager_queue', $queue);

  // Install queued plugins.
  if (!empty($queue) && $clicked == 'Install') {
    drupal_goto('admin/config/system/plugin_manager/install');
  }

}

/**
 * FAPI page for selecting plugin versions.
 */
function plugin_manager_versions_form() {
  $queue_info = variable_get('plugin_manager_queue_info', array());

  // Add a version drop down for each of the queued projects.
  foreach ($queue_info as $project => $info) {
    $form['version_' . $project] = array(
      '#title' => t('%title version', array('%title' => $info['title'])),
      '#type' => 'select',
      '#options' => $info['version'],
    );
  }

  // Add the Continue submit button.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Continue'),
  );
  return $form;
}

/**
 * Plugin Manager versions form validation handler.
 */
function plugin_manager_versions_form_validate($form, &$form_state) {
  $queue = variable_get('plugin_manager_queue', array());
  if (empty($queue)) {
    form_set_error('versions', t('No plugins in the queue.'));
  }
}

/**
 * Plugin Manager versions form submit handler.
 */
function plugin_manager_versions_form_submit($form, &$form_state) {
  $versions = array();
  $queue = variable_get('plugin_manager_queue', array());
  $queue_info = plugin_manager_get_release_history($queue);
  foreach ($form_state['values'] as $element => $version) {
    if (substr($element, 0, 8) == 'version_') {
      $project = substr($element, 8);
      $versions[$project] = $queue_info[$project]['release'][$version]['version'];
    }
  }
  variable_set('plugin_manager_versions', $versions);
  drupal_goto('admin/config/system/plugin_manager/install');
}

/**
 * A page to list all of the queued changes and begin installation of them.
 */
function plugin_manager_install_form($form_state) {
  // Redirect to the version selection form if not selecting the latest versions.
  $latest = variable_get('plugin_manager_latest_version', 1);
  $versions = variable_get('plugin_manager_versions', array());
  if (empty($versions)) {
    if (!$latest) {
      drupal_goto('admin/config/system/plugin_manager/versions');
    }
  }

  // If there aren't any plugins selected, then just mention that and quit.
  $queue = variable_get('plugin_manager_queue', array());
  if (empty($queue)) {
    drupal_set_message(t('Nothing has been selected to install. !search.', array(
      '!search' => l(t('Search for modules or themes to install'), 'admin/config/system/plugin_manager'),
    )));
    return array();
  }

  // @todo Examine this code.
  // Get the total list and grab the release_history for each and
  // set the download links.
  $queue_info = plugin_manager_get_release_history($queue);
  variable_set('plugin_manager_queue_info', $queue_info);

  // If we don't need to select a version, then skip that page.
  $form = drupal_get_form('update_manager_install_form');
  module_load_include('inc', 'update', 'update.manager');
  $key = 0;
  $form_state = array();
  foreach ($queue_info as $name => $plugin) {
    if (!$latest) {
      foreach ($plugin['release'] as $index => $release) {
        if ($release['version'] == $versions[$name]) {
          $key = $index;
          break;
        }
      }
    }
    $context = db_query('SELECT context FROM {plugin_manager_repository} WHERE short_name = :name', array(':name' => $name))->fetchField();
    $link = $plugin['release'][$key]['download_link'];
    $version = $plugin['release'][$key]['version'];
    $form_state['values']['project_url'] = $link;
    $form_state['values']['op'] = 'Install';
    $form_state['build_info']['args'] = array($context);
    //$form_state['redirect'] = FALSE;
    $form_state['submit'] = array('plugin_manager_update_manager_install_form_submit');
    //$form_state['rebuild'] = TRUE;
    drupal_form_submit('update_manager_install_form', $form_state);
    drupal_set_message(t('Added the :plugin !context.', array(':plugin' => $name, '!context' => $context)), 'status', FALSE);
  }
}

/**
 * Form submission handler for update_manager_install_form().
 *
 * Either downloads the file specified in the URL to a temporary cache, or
 * uploads the file attached to the form, then attempts to extract the archive
 * into a temporary location and verify it. Instantiate the appropriate
 * Updater class for this project and make sure it is not already installed in
 * the live webroot. If everything is successful, setup an operation to run
 * via authorize.php which will copy the extracted files from the temporary
 * location into the live site.
 *
 * @see update_manager_install_form_validate()
 * @see update_authorize_run_install()
 * @see system_authorized_init()
 * @see system_authorized_get_url()
 */
function plugin_manager_update_manager_install_form_submit($form, &$form_state) {
  if ($form_state['values']['project_url']) {
    $field = 'project_url';
    $local_cache = update_manager_file_get($form_state['values']['project_url']);
    if (!$local_cache) {
      form_set_error($field, t('Unable to retrieve Drupal project from %url.', array('%url' => $form_state['values']['project_url'])));
      return;
    }
  }
  elseif ($_FILES['files']['name']['project_upload']) {
    $validators = array('file_validate_extensions' => array(archiver_get_extensions()));
    $field = 'project_upload';
    if (!($finfo = file_save_upload($field, $validators, NULL, FILE_EXISTS_REPLACE))) {
      // Failed to upload the file. file_save_upload() calls form_set_error() on
      // failure.
      return;
    }
    $local_cache = $finfo->uri;
  }

  $directory = _update_manager_extract_directory();
  try {
    $archive = update_manager_archive_extract($local_cache, $directory);
  }
  catch (Exception $e) {
    form_set_error($field, $e->getMessage());
    return;
  }

  $files = $archive->listContents();
  if (!$files) {
    form_set_error($field, t('Provided archive contains no files.'));
    return;
  }

  // Unfortunately, we can only use the directory name to determine the project
  // name. Some archivers list the first file as the directory (i.e., MODULE/)
  // and others list an actual file (i.e., MODULE/README.TXT).
  $project = strtok($files[0], '/\\');

  $archive_errors = update_manager_archive_verify($project, $local_cache, $directory);
  if (!empty($archive_errors)) {
    form_set_error($field, array_shift($archive_errors));
    // @todo: Fix me in D8: We need a way to set multiple errors on the same
    // form element and have all of them appear!
    if (!empty($archive_errors)) {
      foreach ($archive_errors as $error) {
        drupal_set_message($error, 'error');
      }
    }
    return;
  }

  // Make sure the Updater registry is loaded.
  drupal_get_updaters();

  $project_location = $directory . '/' . $project;
  try {
    $updater = Updater::factory($project_location);
  }
  catch (Exception $e) {
    form_set_error($field, $e->getMessage());
    return;
  }

  try {
    $project_title = Updater::getProjectTitle($project_location);
  }
  catch (Exception $e) {
    form_set_error($field, $e->getMessage());
    return;
  }

  if (!$project_title) {
    form_set_error($field, t('Unable to determine %project name.', array('%project' => $project)));
  }

  if ($updater->isInstalled()) {
    form_set_error($field, t('%project is already installed.', array('%project' => $project_title)));
    return;
  }

  $project_real_location = drupal_realpath($project_location);
  $arguments = array(
    'project' => $project,
    'updater_name' => get_class($updater),
    'local_url' => $project_real_location,
  );

  // If the owner of the directory we extracted is the same as the
  // owner of our configuration directory (e.g. sites/default) where we're
  // trying to install the code, there's no need to prompt for FTP/SSH
  // credentials. Instead, we instantiate a FileTransferLocal and invoke
  // update_authorize_run_install() directly.
  if (fileowner($project_real_location) == fileowner(conf_path())) {
    module_load_include('inc', 'update', 'update.authorize');
    $filetransfer = new FileTransferLocal(DRUPAL_ROOT);
    call_user_func_array('update_authorize_run_install', array_merge(array($filetransfer), $arguments));
  }
  // Otherwise, go through the regular workflow to prompt for FTP/SSH
  // credentials and invoke update_authorize_run_install() indirectly with
  // whatever FileTransfer object authorize.php creates for us.
  else {
    //system_authorized_init('update_authorize_run_install', drupal_get_path('module', 'update') . '/update.authorize.inc', $arguments, t('Update manager'));
    //$form_state['redirect'] = system_authorized_get_url();
  }
}

/**
 * Plugin manager settings form.
 */
function plugin_manager_settings_form() {
  $form = array();

  $form['plugin_manager_latest_version'] = array(
    '#type' => 'checkbox',
    '#title' => t('Choose Latest Version'),
    '#description' => t('If this is selected, the plugin manager will skip version selection and automatically select the most recent version.'),
    '#default_value' => variable_get('plugin_manager_latest_version', 1),
  );

  $form[] = array(
    '#type' => 'submit',
    '#value' => t('Save Settings'),
  );

  return $form;
}

/**
 * Plugin manager settings form submit handler.
 */
function plugin_manager_settings_form_submit($form, &$form_state) {
  foreach ($form_state['values'] as $name => $value) {
    if (!strcmp(drupal_substr($name, 0, drupal_strlen('plugin_manager_')), 'plugin_manager_')) {
      variable_set($name, $value);
    }
  }
  drupal_set_message(t('The settings have been saved.'));
}
