<?php

/**
 * @file
 * The SSH Backend for the plugin manager.
 */

/**
 * Install the supplied files to the appropriate locations.
 *
 * @param string $files
 *   The files to install.
 * @param string $type
 *   Either 'module' or 'theme' to indicate where to install to.
 * @param string $host
 *   Server (usually localhost).
 * @param string $username
 *   Username.
 * @param string $password
 *   Password.
 *
 * @return bool
 *   TRUE if all the files were installed, FALSE otherwise.
 */
function ssh_plugin_manager_copy($files, $type, $host, $username, $password) {
  // Figure out the type of files.
  if (!in_array($type, array('modules', 'themes'))) {
    return FALSE;
  }
  $dir = 'sites/all/' . drupal_strtolower($type);

  // Connect to the server via ssh, parsing for non-standard port.
  $parts = explode(':', $host);
  if (count($parts) == 2) {
    $connection = ssh2_connect($parts[0], $parts[1] + 0);
  }
  else {
    $connection = ssh2_connect($host);
  }

  if ($connection == FALSE) {
    drupal_set_message(t('Could not connect to ssh on this host.'), 'error');
    return FALSE;
  }

  if (!ssh2_auth_password($connection, $username, $password)) {
    drupal_set_message(t('The supplied username/password combination was not accepted.'), 'error');
    return FALSE;
  }

  // Prepare the directories to use later.
  $extract_dir = variable_get('file_' . file_default_scheme() . '_path', conf_path() . '/files') . '/plugin_manager_extraction';

  // Try to guess the ftp address for the drupal install.
  // Start by putting the entire address together and replacing \ with /.
  $local_path = rtrim(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT'] . base_path()), '/');

  // Process each of the files.
  foreach ($files as $file) {
    if (trim($file, "\\/") != $file) {
      ssh2_exec($connection, 'mkdir ' . escapeshellarg("$local_path/$dir/$file"));
    }
    else {
      ssh2_scp_send($connection, "$extract_dir/$file", "$local_path/$dir/$file");
    }
  }

  return TRUE;
}

/**
 * Remove the supplied files.
 *
 * @param string $files
 *   The files to remove.
 * @param string $host
 *   Server (usually localhost).
 * @param string $username
 *   Username.
 * @param string $password
 *   Password.
 *
 * @return bool
 *   TRUE if all the files were installed, FALSE otherwise.
 */
function ssh_plugin_manager_remove($files, $type, $host, $username, $password) {
  $dir = 'sites/all';

  // Connect to the server via ssh, parsing for non-standard port.
  $parts = explode(':', $host);
  if (count($parts) == 2) {
    $connection = ssh2_connect($parts[0], $parts[1] + 0);
  }
  else {
    $connection = ssh2_connect($host);
  }

  if ($connection == FALSE) {
    drupal_set_message(t('Could not connect to ssh on this host.'), 'error');
    return FALSE;
  }

  if (!ssh2_auth_password($connection, $username, $password)) {
    drupal_set_message(t('The supplied username/password combination was not accepted.'), 'error');
    return FALSE;
  }

  // Try to guess the ftp address for the drupal install.
  // Start by putting the entire address together and replacing \ with /.
  $local_path = rtrim(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT'] . base_path()), '/');

  // Process each of the files.
  foreach ($files as $file) {
    if (trim($file, "\\/") != $file) {
      ssh2_exec($connection, 'rmdir ' . escapeshellarg("$local_path/$dir/$file"));
    }
    else {
      ssh2_exec($connection, 'rm ' . escapeshellarg("$local_path/$dir/$file"));
    }
  }

  return TRUE;
}
